@@ -30,114 +30,56 @@ For news, bugfixes, etc. visit the home page for this implementation at
     http://www.wayforward.net/spf/
 """
 
-# CVS Commits since last release (2.0.7):
+# CVS Commits since last release (2.0.8):
 # $Log: spf.py,v $
-# Revision 1.108.2.109  2013/07/25 01:51:24  customdesigned
-# Forgot to convert to bytes in py3dns-3.0.2 workaround.
+# Revision 1.108.2.122  2014/04/29 22:56:48  customdesigned
+# Release 2.0.9
 #
-# Revision 1.108.2.108  2013/07/25 01:29:07  customdesigned
-# The Final and Ultimate Solution to the String Problem for TXT records.
+# Revision 1.108.2.121  2014/04/28 21:57:08  customdesigned
+# Ignore void lookups for explanation and type 99 lookup.
 #
-# Revision 1.108.2.107  2013/07/23 18:37:17  customdesigned
-# Removed decode from dns_txt again, as it breaks python3, both with py3dns and test framework.
-# Need to identify exact situation in which it is needed to put it back.
+# Revision 1.108.2.120  2014/04/24 23:02:15  kitterma
+# Remove redundant check of self.void_lookups.
 #
-# Revision 1.108.2.106  2013/07/23 06:32:58  kitterma
-# Post fix cleanup.
+# Revision 1.108.2.119  2014/04/22 23:03:42  kitterma
+# Update CHANGELOG to prepare for release.
 #
-# Revision 1.108.2.105  2013/07/23 06:30:13  kitterma
-# Fix compatibility with py3dns versions that return type bytes.
+# Revision 1.108.2.118  2014/04/22 22:03:13  kitterma
+# Add processing for new void lookups processing limit.
 #
-# Revision 1.108.2.104  2013/07/23 06:20:18  kitterma
-# Consolidate code related to UnicodeDecodeError and UnicodeEncodeError into UnicodeError.
+# Revision 1.108.2.117  2014/04/22 20:54:42  kitterma
+# Adjust documentation of lookup limits to include RFC 7208
+# Add constants and variables for new void lookup limit
 #
-# Revision 1.108.2.103  2013/07/23 06:07:24  customdesigned
-# Test case and fix for allowing non-ascii in non-spf TXT records.
+# Revision 1.108.2.116  2014/04/22 17:10:54  kitterma
+# Default SPF process timeout limit to 20 seconds per RFC 7208 4.6.4.
 #
-# Revision 1.108.2.102  2013/07/23 05:22:54  customdesigned
-# Check for non-ascii on explanation.
+# Revision 1.108.2.115  2014/04/22 17:02:55  kitterma
+# Change default DNS timeout to 20 seconds in DNSLookup to better match RFC
+# 7208 4.6.4.
 #
-# Revision 1.108.2.101  2013/07/23 04:51:59  customdesigned
-# Functional alias for __email__
+# Revision 1.108.2.114  2014/04/22 04:56:38  kitterma
+# Add permerror to permitted mx-limit results for rfc4408 to fudge changes for
+# RFC 7208.
 #
-# Revision 1.108.2.100  2013/07/23 04:07:38  customdesigned
-# Sort unofficial keywords for consistent ordering.
+# Revision 1.108.2.113  2014/04/22 04:46:58  kitterma
+# Make mx > 10 a permerror per RFC 7208 and mx-limit test.
 #
-# Revision 1.108.2.99  2013/07/23 02:40:54  customdesigned
-# Update __email__ and __author__
+# Revision 1.108.2.112  2014/01/20 22:16:38  customdesigned
+# Rename local var hiding str.
 #
-# Revision 1.108.2.98  2013/07/23 02:35:33  customdesigned
-# Release 2.0.8
+# Revision 1.108.2.111  2014/01/20 22:03:08  customdesigned
+# Test case and fix for more thorough macro syntax error detection.
 #
-# Revision 1.108.2.97  2013/07/23 02:04:59  customdesigned
-# Release 2.0.8
+# Revision 1.108.2.110  2013/07/25 21:21:49  kitterma
+# Archive previous commit messages for spf.py in pyspf_changelog.txt and bump version to 2.0.9 for start of follow on work.
 #
-# Revision 1.108.2.96  2013/07/22 22:59:58  kitterma
-# Give another header test it's own variable names.
-#
-# Revision 1.108.2.95  2013/07/22 19:29:22  kitterma
-# Fix dns_txt to work if DNS data is not pure bytes for python3 compatibility.
-#
-# Revision 1.108.2.94  2013/07/22 02:44:39  kitterma
-# Add tests for cirdmatch.
-#
-# Revision 1.108.2.93  2013/07/21 23:56:51  kitterma
-# Fix cidrmatch to work with both ipaddr and the python3.3 ipadrress versions of the module.
-#
-# Revision 1.108.2.91  2013/07/03 23:38:39  customdesigned
-# Removed two more unused functions.
-#
-# Revision 1.108.2.90  2013/07/03 22:58:26  customdesigned
-# Clean up use of ipaddress module.  make %{i} upper case to match test suite
-# (test suite is incorrect requiring uppercase, but one thing at a time).
-# Remove no longer used inet_pton substitute.  But what if someone was using it?
-#
-# Revision 1.108.2.89  2013/05/26 03:32:19  kitterma
-# Syntax fix to maintain python2.6 compatibility.
-#
-# Revision 1.108.2.88  2013/05/26 00:30:12  kitterma
-# Bump versions to 2.0.8 and add CHANGELOG entries.
-#
-# Revision 1.108.2.87  2013/05/26 00:23:52  kitterma
-# Move old (pre-2.0.7) spf.py commit messages to pyspf_changelog.txt.
-#
-# Revision 1.108.2.86  2013/05/25 22:39:19  kitterma
-# Use ipaddr/ipaddress instead of custome code.
-#
-# Revision 1.108.2.85  2013/05/25 00:06:03  kitterma
-# Fix return type detection for bytes/string for python3 compatibility in dns_txt.
-#
-# Revision 1.108.2.84  2013/04/20 20:49:13  customdesigned
-# Some dual-cidr doc tests
-#
-# Revision 1.108.2.83  2013/03/25 22:51:37  customdesigned
-# Replace dns_99 method with dns_txt(type='SPF')
-# Fix null CNAME in cache bug.
-#
-# Revision 1.108.2.82  2013/03/14 21:13:06  customdesigned
-# Fix Non-ascii exception description.
-#
-# Revision 1.108.2.81  2013/03/14 21:03:25  customdesigned
-# Fix dns_txt and dns_spf - should hopefully still be correct for python3.
-#
-# Revision 1.108.2.80  2012/06/14 20:09:56  kitterma
-# Use the correct exception type to capture unicode in SPF records.
-#
-# Revision 1.108.2.79  2012/03/10 00:19:44  kitterma
-# Add fixes for py3dns DNS return as type bytes - not complete.
-#
-# Revision 1.108.2.77  2012/02/09 22:13:42  kitterma
-# Fix stray character in last commit.
-# Start fixing python3 bytes issue - Now works, but fails the non-ASCII exp test.
-#
-# Revision 1.108.2.76  2012/02/05 05:50:39  kitterma
-# Fix a few stray print -> print() changes for python3 compatbility.
 #
 # See pyspf_changelog.txt for earlier CVS commits.
 
 __author__ = "Terence Way, Stuart Gathman, Scott Kitterman"
 __email__ = "pyspf@openspf.org"
-__version__ = "2.0.8: Jul 22, 2013"
+__version__ = "2.0.9: Apr 29, 2014"
 MODULE = 'spf'
 
 USAGE = """To check an incoming mail request:
@@ -187,7 +129,7 @@ if not hasattr(DNS.Type, 'SPF'):
     DNS.Type.typemap[99] = 'SPF'
     DNS.Lib.RRunpacker.getSPFdata = DNS.Lib.RRunpacker.getTXTdata
 
-def DNSLookup(name, qtype, strict=True, timeout=30):
+def DNSLookup(name, qtype, strict=True, timeout=20):
     try:
         req = DNS.DnsRequest(name, qtype=qtype, timeout=timeout)
         resp = req.req()
@@ -224,6 +166,7 @@ RE_MODIFIER = re.compile(r'^([a-z][a-z0-9_\-\.]*)=', re.IGNORECASE)
 # Regular expression to find macro expansions
 PAT_CHAR = r'%(%|_|-|(\{[^\}]*\}))'
 RE_CHAR = re.compile(PAT_CHAR)
+RE_INVALID_MACRO = re.compile(r'(?<!%)%[^{%_-]|%$')
 
 # Regular expression to break up a macro expansion
 RE_ARGS = re.compile(r'([0-9]*)(r?)([^0-9a-zA-Z]*)')
@@ -291,12 +234,13 @@ DEFAULT_SPF = 'v=spf1 a/24 mx/24 ptr'
 TRUSTED_FORWARDERS = 'v=spf1 ?include:spf.trusted-forwarder.org -all'
 
 # maximum DNS lookups allowed
-MAX_LOOKUP = 10 #RFC 4408 Para 10.1
-MAX_MX = 10 #RFC 4408 Para 10.1
-MAX_PTR = 10 #RFC 4408 Para 10.1
+MAX_LOOKUP = 10 #RFC 4408 Para 10.1/RFC 7208 4.6.4
+MAX_MX = 10 #RFC 4408 Para 10.1/RFC 7208 4.6.4
+MAX_PTR = 10 #RFC 4408 Para 10.1/RFC 7208 4.6.4
 MAX_CNAME = 10 # analogous interpretation to MAX_PTR
 MAX_RECURSION = 20
-MAX_PER_LOOKUP_TIME = 30 # Long standing pyspf default
+MAX_PER_LOOKUP_TIME = 20 # Per RFC 7208 4.6.4
+MAX_VOID_LOOKUPS = 2 # RFC 7208 4.6.4
 
 ALL_MECHANISMS = ('a', 'mx', 'ptr', 'exists', 'include', 'ip4', 'ip6', 'all')
 COMMON_MISTAKES = {
@@ -341,16 +285,19 @@ class PermError(Exception):
             return '%s: %s'%(self.msg, self.mech)
         return self.msg
 
-def check2(i, s, h, local=None, receiver=None, timeout=MAX_PER_LOOKUP_TIME, verbose=False, querytime=0):
+def check2(i, s, h, local=None, receiver=None, timeout=MAX_PER_LOOKUP_TIME, verbose=False, querytime=20):
     """Test an incoming MAIL FROM:<s>, from a client with ip address i.
-    h is the HELO/EHLO domain name.  This is the RFC4408 compliant pySPF2.0
-    interface.  The interface returns an SPF result and explanation only.
-    SMTP response codes are not returned since RFC 4408 does not specify
-    receiver policy.  Applications updated for RFC 4408 should use this
-    interface.  The maximum time, in seconds, this function is allowed to run
-    before a TempError is returned is controlled by querytime.  When set to 0
-    (default) the timeout parameter (default 30 seconds) controls the time
-    allowed for each DNS lookup.
+    h is the HELO/EHLO domain name.  This is the RFC4408/7208 compliant
+    pySPF2.0 interface.  The interface returns an SPF result and explanation
+    only.  SMTP response codes are not returned since neither RFC 4408 nor RFC
+    7208 does specify receiver policy.  Applications updated for RFC 4408 and
+    RFC 7208 should use this interface.  The maximum time, in seconds, this
+    function is allowed to run before a TempError is returned is controlled by
+    querytime.  When set to 0 the timeout parameter (default 20 seconds)
+    controls the time allowed for each DNS lookup.  When set to a non-zero
+    value, it total time for all processing related to the SPF check is
+    limited to querytime (default 20 seconds as recommended in RFC 7208,
+    paragraph 4.6.4).
 
     Returns (result, explanation) where result in
     ['pass', 'permerror', 'fail', 'temperror', 'softfail', 'none', 'neutral' ].
@@ -430,6 +377,8 @@ class query(object):
         self.exps = dict(EXPLANATIONS)
         self.libspf_local = local    # local policy
         self.lookups = 0
+        # New processing limit in RFC 7208, section 4.6.4
+        self.void_lookups = 0
         # strict can be False, True, or 2 (numeric) for harsh
         self.strict = strict
         self.timeout = timeout
@@ -602,7 +551,6 @@ class query(object):
 
         try:
             self.lookups = 0
-            self.timer = 0
             if not spf:
                 spf = self.dns_spf(self.d)
                 if self.verbose: self.log("top",self.d,spf)
@@ -1014,7 +962,7 @@ class query(object):
         """Expand an explanation."""
         if spec:
             try:
-                a = self.dns_txt(spec)
+                a = self.dns_txt(spec,ignore_void=True)
                 if len(a) == 1:
                     return str(self.expand(to_ascii(a[0]), stripdot=False))
             except PermError:
@@ -1028,7 +976,7 @@ class query(object):
         # (unless you give precedence to the grammar).
         return None
 
-    def expand(self, str, stripdot=True): # macros='slodipvh'
+    def expand(self, s, stripdot=True): # macros='slodipvh'
         """Do SPF RFC macro expansion.
 
         Examples:
@@ -1114,25 +1062,18 @@ class query(object):
         'postmaster'
 
         """
-        macro_delimiters = ['{', '%', '-', '_']
+        # Check for invalid macro syntax
+        if s.find('%') >= 0:
+            regex = RE_INVALID_MACRO
+            for label in s.split('.'):
+                if regex.search(s):
+                    raise PermError ('invalid-macro-char ', label)
+        # expand macros
         end = 0
         result = ''
-        macro_count = str.count('%')
-        if macro_count != 0:
-            labels = str.split('.')
-            for label in labels:
-                is_macro = False
-                if len(label) > 1:
-                    if label[0] == '%':
-                        for delimit in macro_delimiters:
-                            if label[1] == delimit:
-                                is_macro = True
-                        if not is_macro:
-                            raise PermError ('invalid-macro-char ', label)
-                            break
-        for i in RE_CHAR.finditer(str):
-            result += str[end:i.start()]
-            macro = str[i.start():i.end()]
+        for i in RE_CHAR.finditer(s):
+            result += s[end:i.start()]
+            macro = s[i.start():i.end()]
             if macro == '%%':
                 result += '%'
             elif macro == '%_':
@@ -1157,7 +1098,7 @@ class query(object):
                     result += e
 
             end = i.end()
-        result += str[end:]
+        result += s[end:]
         if stripdot and result.endswith('.'):
             result =  result[:-1]
         if result.count('.') != 0:
@@ -1185,7 +1126,8 @@ class query(object):
         if self.strict > 1:
             #Only check for Type SPF in harsh mode until it is more popular.
             try:
-                b = [t for t in self.dns_txt(domain,'SPF') if RE_SPF.match(t)]
+                b = [t for t in self.dns_txt(domain,'SPF',ignore_void=True)
+			if RE_SPF.match(t)]
             except TempError as x:
                 # some braindead DNS servers hang on type 99 query
                 if self.strict > 1: raise TempError(x)
@@ -1202,7 +1144,7 @@ class query(object):
             return to_ascii(a[0])    # return TXT if SPF wasn't found
         if DELEGATE:    # use local record if neither found
             a = [t
-              for t in self.dns_txt(domain+'._spf.'+DELEGATE)
+              for t in self.dns_txt(domain+'._spf.'+DELEGATE,ignore_void=True)
             if RE_SPF.match(t)
             ]
             if len(a) == 1: return to_ascii(a[0])
@@ -1220,11 +1162,11 @@ class query(object):
     # records - even when they are non-ascii.  So we return bytes.
     # The caller does the ascii check for SPF records and explanations.
     # 
-    def dns_txt(self, domainname, rr='TXT'):
+    def dns_txt(self, domainname, rr='TXT',ignore_void=False):
         "Get a list of TXT records for a domain name."
         if domainname:
           try:
-              dns_list = self.dns(domainname, rr)
+              dns_list = self.dns(domainname, rr,ignore_void=ignore_void)
               if dns_list:
                   # a[0][:0] is '' for py3dns-3.0.2, otherwise b''
                   a = [a[0][:0].join(a) for a in dns_list]
@@ -1242,15 +1184,16 @@ class query(object):
         """Get a list of IP addresses for all MX exchanges for a
         domain name.
         """
-        # RFC 4408 section 5.4 "mx"
+        # RFC 4408/7208 section 5.4 "mx"
         # To prevent DoS attacks, more than 10 MX names MUST NOT be looked up
+        # Changed to permerror if more than 10 exist in 7208
         mxnames = self.dns(domainname, 'MX')
         if self.strict:
             max = MAX_MX
+            if len(mxnames) > MAX_MX:
+                raise PermError(
+                    'More than %d MX records returned'%MAX_MX)
             if self.strict > 1:
-                if len(mxnames) > MAX_MX:
-                    raise AmbiguityWarning(
-                        'More than %d MX records returned'%MAX_MX)
                 if len(mxnames) == 0:
                     raise AmbiguityWarning(
                         'No MX records found for mx mechanism', domainname)
@@ -1321,7 +1264,7 @@ class query(object):
     }
 
     # FIXME: move to dnsplug
-    def dns(self, name, qtype, cnames=None):
+    def dns(self, name, qtype, cnames=None, ignore_void=False):
         """DNS query.
 
         If the result is in cache, return that.  Otherwise pull the
@@ -1348,7 +1291,7 @@ class query(object):
         else:
             safe2cache = query.SAFE2CACHE
             if self.querytime < 0:
-                 raise TempError('DNS Error: exceeded max query lookup time')
+                raise TempError('DNS Error: exceeded max query lookup time')
             if self.querytime < self.timeout and self.querytime > 0:
                 timeout = self.querytime
             else:
@@ -1376,6 +1319,10 @@ class query(object):
             result = self.dns(cname, qtype, cnames=cnames)
             if result:
                 self.cache[(name,qtype)] = result
+        if not result and not ignore_void:
+            self.void_lookups += 1
+            if self.void_lookups > MAX_VOID_LOOKUPS:
+                raise PermError('Void lookup limit of %d exceeded' % MAX_VOID_LOOKUPS)
         return result
 
     def cidrmatch(self, ipaddrs, n):
